﻿; EnableExplicit

; LZMA SDK 4.65
; http://www.7-zip.org/sdk.html

; LZMA is default and general compression method of 7z format
; in 7-Zip compression program (www.7-zip.org). LZMA provides high 
; compression ratio and very fast decompression

; LZMA is an improved version of famous LZ77 compression algorithm. 
; It was improved in way of maximum increasing of compression ratio,
; keeping high decompression speed and low memory requirements for 
; decompressing.

; LZMA SDK is written and placed in the public domain by Igor Pavlov.

; Wrapper 1.0 for PB 4.41 by Luis (Feb 2010)

; In brief:
; compression: very good ratio, good speed, very customizable ram requirements (see dictionary size)
; decompression: very good speed, ram requirements depending on the dictionary size used for compression


#LZMA_PROPS_SIZE = 5

; all ok
#LZMA_OK = 0

; error codes for LZMA
#LZMA_ERROR         = 1
#LZMA_ERROR_MEM     = 2
#LZMA_ERROR_OUTPUT  = 3
#LZMA_ERROR_PARAM   = 4
#LZMA_ERROR_DATA    = 5 
#LZMA_ERROR_THREAD  = 6


; dictionary size (1 << #LZMA_DICT_*)
#LZMA_DICT_4K   = 12
#LZMA_DICT_8K   = 13
#LZMA_DICT_16K  = 14
#LZMA_DICT_32K  = 15
#LZMA_DICT_64K  = 16
#LZMA_DICT_128K = 17
#LZMA_DICT_256K = 18
#LZMA_DICT_512K = 19
#LZMA_DICT_1M   = 20
#LZMA_DICT_2M   = 21
#LZMA_DICT_4M   = 22
#LZMA_DICT_8M   = 23
#LZMA_DICT_16M  = 24
#LZMA_DICT_32M  = 25
#LZMA_DICT_64M  = 26
#LZMA_DICT_128M = 27
#LZMA_DICT_256M = 28

Structure T_LZMA_HEADER
 Props.b        ; Special LZMA properties (lc,lp, pb in encoded form)
 DictSize.l     ; Dictionary size 
 SourceSize.l   ; Uncompressed size
EndStructure

CompilerIf (#PB_Compiler_Processor = #PB_Processor_x86)
 Import "libs\" + "lzma_32.lib"
    LzmaCompress.i (*DestBuff, *iDestSize, *SourceBuff, iSourceSize, *OutProps, *iOutPropsSize, iCompressionLevel, iDictSize, Lc, Lp, Pb, Fb, iNumThreads) 
    LzmaUncompress.i (*DestBuff, *iDestSize, *SourceBuff, *iSourceSize, *Props, iPropsSize)
 EndImport
CompilerElse   
 Import "libs\" + "lzma_64.lib"
    LzmaCompress.i (*DestBuff, *iDestSize, *SourceBuff, iSourceSize, *OutProps, *iOutPropsSize, iCompressionLevel, iDictSize, Lc, Lp, Pb, Fb, iNumThreads) 
    LzmaUncompress.i (*DestBuff, *iDestSize, *SourceBuff, *iSourceSize, *Props, iPropsSize)    
 EndImport 
CompilerEndIf


Procedure.i CalcDestBufferSize_LZMA (SourceBuffSize)
 ProcedureReturn (SourceBuffSize + SizeOf(T_LZMA_HEADER))
EndProcedure


 ; *SourceBuff  = ptr to the source buffer
 ; *DestBuff    = ptr to the destination buffer
 ; iSourceSize  = size of the source buffer
 ; *iDestSize   = ptr to the size of the destination buffer (it will contain the packed size on exit)
 
 ; NOTE: The size of the destination buffer *should* be at least the size of the source + SizeOf(T_LZMA_HEADER)
 ;       You can use the procedure CalcDestBufferSize_LZMA() to get it.
 
 ; NOTE: iCompressionLevel must be between 0 (lowest) and 9 (highest) - default 9
 
 ; NOTE: iDictSize must be between 12 (1 << 12 = 4 KB) and 28 (1 << 28 = 256 MB) - default 4MB
 
 ; NOTE: The destination buffer will contain a T_LZMA_HEADER followed by the packed data
 
 ; NOTE: #LZMA_ERROR_OUTPUT_EOF means the file cannot be compressed (or the destination buffer is too small)
Procedure.i PackMemory_LZMA (*SourceBuff, *DestBuff, iSourceSize, *iDestSize.Integer, iCompressionLevel = 9, iDictSize = #LZMA_DICT_4M)
 Protected iResult, iOutPropsSize, *OutProps
 Protected iDestSize = *iDestSize\i - SizeOf(T_LZMA_HEADER)
 
 Protected Lc = 3 
 Protected Lp = 0
 Protected Pb = 2
 Protected Fb = 32
 Protected iNumThreads = 2 ; Mal schauen
 
 iDictSize = 1 << iDictSize ; default 22 = 4 MB
 
 ; Allocate memory for the properties section
 *OutProps = AllocateMemory(#LZMA_PROPS_SIZE) ; 5 bytes, I refuse to check for this!
 
 iOutPropsSize = #LZMA_PROPS_SIZE
 
 iResult = LzmaCompress (*DestBuff + SizeOf(T_LZMA_HEADER), @iDestSize, *SourceBuff, iSourceSize, *OutProps, @iOutPropsSize, iCompressionLevel, iDictSize, Lc,Lp,Pb,Fb, iNumThreads) 
 
 Select iResult
    Case #LZMA_OK 
        ; write the header for the destination buffer
        CopyMemory(*OutProps, *DestBuff, #LZMA_PROPS_SIZE)
        PokeL(*DestBuff + #LZMA_PROPS_SIZE, iSourceSize)
     
        If iDestSize >= iSourceSize
            ; return -1 for the size of the compressed data 
            *iDestSize\i = -1; cannot compress this
            iResult = #LZMA_ERROR_OUTPUT
        Else
            ; return the size of the compressed data starting at *DestBuff
            *iDestSize\i = iDestSize + SizeOf(T_LZMA_HEADER)
        EndIf
    Case 2 ; SZ_ERROR_MEM
        iResult = #LZMA_ERROR_MEM    
    Case 7 ; SZ_ERROR_OUTPUT_EOF
        iResult = #LZMA_ERROR_OUTPUT
    Case 12 ; SZ_ERROR_THREAD
        iResult = #LZMA_ERROR_THREAD        
    Default
        iResult = #LZMA_ERROR           
 EndSelect   
 
 FreeMemory(*OutProps)
    
 ProcedureReturn iResult
EndProcedure

 ; *SourceBuff  = ptr to the source buffer
 ; *DestBuff    = ptr to the destination buffer
 ; iSourceSize  = size of the source buffer
 ; *iDestSize   = ptr to the size of the destination buffer (it will contain the unpacked size on exit)
  
 ; NOTE: The source buffer must contain a T_LZMA_HEADER followed by the packed data
 
 ; NOTE: Memory required for decompression = dictionary size + 16 KB
Procedure.i UnpackMemory_LZMA (*SourceBuff, *DestBuff, iSourceSize, *iDestSize)
 Protected iResult
 Protected iDestSize = PeekI(*iDestSize)
 Protected iOffSourceSize = iSourceSize - SizeOf(T_LZMA_HEADER )
 Protected *Props
   
 ; retrieve Props from header
 *Props = AllocateMemory(#LZMA_PROPS_SIZE) ; 5 bytes
 CopyMemory(*SourceBuff, *Props, #LZMA_PROPS_SIZE)
  
 iResult = LzmaUncompress (*DestBuff, @iDestSize, *SourceBuff + SizeOf(T_LZMA_HEADER), @iOffSourceSize, *Props, #LZMA_PROPS_SIZE)  

 Select iResult
    Case #LZMA_OK 
        PokeI(*iDestSize, iDestSize)
    Case 1, 6 ; SZ_ERROR_DATA, SZ_ERROR_INPUT_EOF
        iResult = #LZMA_ERROR_DATA
    Case 2 ; SZ_ERROR_MEM
        iResult = #LZMA_ERROR_MEM    
    Case 12 ; SZ_ERROR_THREAD
        iResult = #LZMA_ERROR_THREAD        
    Default
        iResult = #LZMA_ERROR           
 EndSelect   
  
 FreeMemory(*Props)
 
 ProcedureReturn iResult
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 63
; FirstLine = 48
; Folding = -
; EnableXP
; EnableUser
; Executable = test.exe
; CPU = 1